import { createAction, props } from '@ngrx/store';

import { LoginSession } from '@ucap/core';
import { Login2Response } from '@ucap/pi';
import {
  LoginRequest,
  LoginResponse,
  LogoutResponse
} from '@ucap/protocol-authentication';
import { UserResponse, UserRequest } from '@ucap/protocol-info';

/**
 * request of web login
 */
export const webLogin = createAction(
  '[ucap::authentication::login] webLogin',
  props<{
    loginId: string;
    loginPw: string;
    companyCode: string;
    rememberMe: boolean;
    autoLogin: boolean;
  }>()
);
/**
 * Success of webLogin request
 */
export const webLoginSuccess = createAction(
  '[ucap::authentication::login] webLogin Success',
  props<{
    loginId: string;
    loginPw: string;
    companyCode: string;
    rememberMe: boolean;
    autoLogin: boolean;
    login2Response: Login2Response;
  }>()
);
/**
 * Failure of webLogin request
 */
export const webLoginFailure = createAction(
  '[ucap::authentication::login] webLogin Failure',
  props<{ error: any }>()
);

/**
 * request of login
 */
export const login = createAction(
  '[ucap::authentication::login] login',
  props<{
    req: LoginRequest;
  }>()
);
/**
 * Success of login request
 */
export const loginSuccess = createAction(
  '[ucap::authentication::login] login Success',
  props<{
    res: LoginResponse;
    loginSession: LoginSession;
  }>()
);
/**
 * Failure of login request
 */
export const loginFailure = createAction(
  '[ucap::authentication::login] login Failure',
  props<{ error: any }>()
);

/**
 * request of logout
 */
export const logout = createAction('[authentication::login] logout');
/**
 * Success of logout request
 */
export const logoutSuccess = createAction(
  '[ucap::authentication::login] logout Success',
  props<{ res: LogoutResponse }>()
);
/**
 * Failure of logout request
 */
export const logoutFailure = createAction(
  '[ucap::authentication::login] logout Failure',
  props<{ error: any }>()
);

/**
 * session created
 */
export const sessionCreated = createAction(
  '[ucap::authentication::login] session Created',
  props<{
    res: LoginResponse;
    loginSession: LoginSession;
  }>()
);
/**
 * session destroyed
 */
export const sessionDestroyed = createAction(
  '[ucap::authentication::login] session Destroyed',
  props<{ error: any }>()
);

/**
 * info user request
 */
export const infoUser = createAction(
  '[ucap::authentication::login] Info User',
  props<{ req: UserRequest }>()
);

/**
 * Success of info user request
 */
export const infoUserSuccess = createAction(
  '[ucap::authentication::login] Info User Success',
  props<{
    res: UserResponse;
  }>()
);

/**
 * Failure of info user request
 */
export const infoUserFailure = createAction(
  '[ucap::authentication::login] Info User Failure',
  props<{ error: any }>()
);
