import { of } from 'rxjs';
import { catchError, map, switchMap, exhaustMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { PiService } from '@ucap/ng-pi';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';
import { InfoProtocolService } from '@ucap/ng-protocol-info';

import {
  logout,
  logoutSuccess,
  infoUser,
  infoUserSuccess,
  infoUserFailure
} from './actions';
import { UserResponse } from '@ucap/protocol-info';

@Injectable()
export class Effects {
  logout$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(logout),
      switchMap((action) => {
        return this.authenticationProtocolService.logout({}).pipe(
          map((res) => logoutSuccess({ res })),
          catchError((error) => of(error))
        );
      })
    );
  });

  infoUser$ = createEffect(() =>
    this.actions$.pipe(
      ofType(infoUser),
      map((action) => action.req),
      exhaustMap((req) =>
        this.infoProtocolService.user(req).pipe(
          map((res: UserResponse) => {
            return infoUserSuccess({
              res
            });
          }),
          catchError((error) => of(infoUserFailure({ error })))
        )
      )
    )
  );

  constructor(
    private actions$: Actions,
    private piService: PiService,
    private authenticationProtocolService: AuthenticationProtocolService,
    private infoProtocolService: InfoProtocolService
  ) {}
}
