import { createReducer, on } from '@ngrx/store';

import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo
} from '@ucap/protocol-room';

import * as chattingActions from '../chatting/actions';

import {
  initialState,
  adapterRoom,
  adapterRoomUser,
  RoomUserMap,
  RoomUserShortMap,
  adapterRoomUserShort
} from './state';
import {
  roomsSuccess,
  roomSuccess,
  excludeUser,
  excludeUserSuccess,
  delSuccess,
  rooms2Success,
  delMultiSuccess,
  updateSuccess,
  room2Success,
  createSuccess
} from './actions';
import { Info, EventJson } from '@ucap/protocol-event';
import { ChatUtil } from '../../utils/chat.util';

export const reducer = createReducer(
  initialState,

  on(roomsSuccess, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const userInfoList = action.roomUserInfoMap[key];
        if (!!userInfoList && 0 < userInfoList.length) {
          roomUserMapList.push({ roomId: key, userInfos: userInfoList });
        }
      }
    }

    let unreadTotal = 0;

    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
    }
    action.roomList.map((item) => (unreadTotal += item.noReadCnt));

    return {
      ...state,
      rooms: adapterRoom.upsertMany(action.roomList, {
        ...state.rooms,
        syncDate: action.syncDate
      }),
      roomUsers: adapterRoomUser.upsertMany(roomUserMapList, {
        ...state.roomUsers
      }),
      unreadTotal
    };
  }),

  on(rooms2Success, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];
    const roomUserShortMapList: RoomUserShortMap[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const userInfosMap = action.roomUserInfoMap[key];
        if (!!userInfosMap && 0 < userInfosMap.userInfoList.length) {
          roomUserMapList.push({
            roomId: key,
            userInfos: userInfosMap.userInfoList
          });
        }
        if (!!userInfosMap && 0 < userInfosMap.userInfoShortList.length) {
          roomUserShortMapList.push({
            roomId: key,
            userInfos: userInfosMap.userInfoShortList
          });
        }
      }
    }

    let unreadTotal = 0;

    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
    }
    action.roomList.map((item) => (unreadTotal += item.noReadCnt));

    return {
      ...state,
      rooms: adapterRoom.upsertMany(action.roomList, {
        ...state.rooms,
        syncDate: action.syncDate
      }),
      roomUsers: adapterRoomUser.upsertMany(roomUserMapList, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.upsertMany(roomUserShortMapList, {
        ...state.roomUsersShort
      }),
      unreadTotal
    };
  }),

  on(roomSuccess, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];

    if (!!action.userInfoList && 0 < action.userInfoList.length) {
      roomUserMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.userInfoList
      });
    }

    return {
      ...state,
      rooms: adapterRoom.upsertOne(action.roomInfo, {
        ...state.rooms
      }),
      roomUsers:
        0 < roomUserMapList.length
          ? adapterRoomUser.upsertMany(roomUserMapList, {
              ...state.roomUsers
            })
          : state.roomUsers
    };
  }),

  on(room2Success, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];
    const roomUserShortMapList: RoomUserShortMap[] = [];

    if (
      !!action.roomUserInfo.userInfoList &&
      0 < action.roomUserInfo.userInfoList.length
    ) {
      roomUserMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.roomUserInfo.userInfoList
      });
    }
    if (
      !!action.roomUserInfo.userInfoShortList &&
      0 < action.roomUserInfo.userInfoShortList.length
    ) {
      roomUserShortMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.roomUserInfo.userInfoShortList
      });
    }

    return {
      ...state,
      rooms: adapterRoom.upsertOne(action.roomInfo, {
        ...state.rooms
      }),
      roomUsers:
        0 < roomUserMapList.length
          ? adapterRoomUser.upsertMany(roomUserMapList, {
              ...state.roomUsers
            })
          : state.roomUsers,
      roomUsersShort:
        0 < roomUserShortMapList.length
          ? adapterRoomUserShort.upsertMany(roomUserShortMapList, {
              ...state.roomUsersShort
            })
          : state.roomUsersShort
    };
  }),

  on(excludeUserSuccess, (state, action) => {
    const roomId = action.roomId;
    const roomUserMap = state.roomUsers.entities[roomId];
    const roomUserMapShort = state.roomUsersShort.entities[roomId];

    const userInfos: RoomUserInfo[] = [...roomUserMap.userInfos];
    const userInfosShort: RoomUserInfoShort[] = [...roomUserMapShort.userInfos];

    action.userSeqs.forEach((userSeq) => {
      const userInfo: RoomUserInfo = userInfos.find(
        (u) => userSeq === String(u.seq)
      );

      if (!!userInfo && !!userInfo.seq) {
        userInfo.isJoinRoom = false;
      }

      const userInfoShort: RoomUserInfoShort = userInfosShort.find(
        (u) => userSeq === String(u.seq)
      );

      if (!!userInfoShort && !!userInfoShort.seq) {
        userInfoShort.isJoinRoom = false;
      }
    });

    return {
      ...state,
      roomUsers: adapterRoomUser.updateOne(
        {
          id: roomId,
          changes: {
            roomId,
            userInfos
          }
        },
        {
          ...state.roomUsers
        }
      ),
      roomUsersShort: adapterRoomUserShort.updateOne(
        {
          id: roomId,
          changes: {
            roomId,
            userInfos: userInfosShort
          }
        },
        {
          ...state.roomUsersShort
        }
      )
    };
  }),

  on(createSuccess, (state, action) => {
    const standby = state.standbyRooms;
    const curRoomId = action.res.roomId;
    if (standby.findIndex((roomId) => roomId === curRoomId) > -1) {
      // Exist.
      return state;
    } else {
      // Not Exist.
      return {
        ...state,
        standbyRooms: [...state.standbyRooms, curRoomId]
      };
    }
  }),

  on(delSuccess, (state, action) => {
    const roomId = action.res.roomId;
    const room = state.rooms.entities[roomId];

    if (!room) {
      return state;
    }

    return {
      ...state,
      rooms: adapterRoom.removeOne(roomId, { ...state.rooms }),
      roomUsers: adapterRoomUser.removeOne(roomId, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.removeOne(roomId, {
        ...state.roomUsersShort
      })
    };
  }),

  on(updateSuccess, (state, action) => {
    const roomInfo = {
      ...state.rooms.entities[action.res.roomId],
      roomName: action.res.roomName,
      receiveAlarm: action.res.receiveAlarm
    } as RoomInfo;

    return {
      ...state,
      rooms: adapterRoom.upsertOne(roomInfo, { ...state.rooms })
    };
  }),

  on(delSuccess, (state, action) => {
    const roomId = action.res.roomId;
    const room = state.rooms.entities[roomId];

    if (!room) {
      return state;
    }

    return {
      ...state,
      rooms: adapterRoom.removeOne(roomId, { ...state.rooms }),
      roomUsers: adapterRoomUser.removeOne(roomId, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.removeOne(roomId, {
        ...state.roomUsersShort
      })
    };
  }),

  on(delMultiSuccess, (state, action) => {
    const roomIds: string[] = action.res.roomIds;

    if (!roomIds || roomIds.length === 0) {
      return state;
    }

    return {
      ...state,
      rooms: adapterRoom.removeMany(roomIds, { ...state.rooms }),
      roomUsers: adapterRoomUser.removeMany(roomIds, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.removeMany(roomIds, {
        ...state.roomUsersShort
      })
    };
  }),

  /*******************************************************************
   * [Chatting Action watching.]
   *******************************************************************/
  on(chattingActions.readSuccess, (state, action) => {
    const roomId = action.roomId;
    const trgtUserSeq = action.SENDER_SEQ;

    if (!roomId || !trgtUserSeq) {
      return state;
    }

    let roomUser: RoomUserMap = state.roomUsers.entities[roomId];
    if (!!roomUser) {
      roomUser = {
        ...roomUser,
        userInfos: state.roomUsers.entities[roomId].userInfos.map(
          (roomUserInfo) => {
            if (roomUserInfo.seq === Number(trgtUserSeq)) {
              return {
                ...roomUserInfo,
                lastReadEventSeq: action.lastReadSeq
              };
            } else {
              return roomUserInfo;
            }
          }
        )
      };
    }
    let roomUserShort: RoomUserShortMap = state.roomUsersShort.entities[roomId];
    if (!!roomUserShort) {
      roomUserShort = {
        ...roomUserShort,
        userInfos: state.roomUsersShort.entities[roomId].userInfos.map(
          (roomUserInfo) => {
            if (roomUserInfo.seq === Number(trgtUserSeq)) {
              return {
                ...roomUserInfo,
                lastReadEventSeq: action.lastReadSeq
              };
            } else {
              return roomUserInfo;
            }
          }
        )
      };
    }

    return {
      ...state,
      roomUsers: !!roomUser
        ? adapterRoomUser.upsertOne(roomUser, {
            ...state.roomUsers
          })
        : state.roomUsers,
      roomUsersShort: !!roomUserShort
        ? adapterRoomUserShort.upsertOne(roomUserShort, {
            ...state.roomUsersShort
          })
        : state.roomUsersShort
    };
  }),

  on(chattingActions.addEventSuccess, (state, action) => {
    const roomId = action.roomId;
    const info: Info<EventJson> = action.info;

    if (!roomId || !info) {
      return state;
    }

    const currentRoomInfo = state.rooms.entities[roomId];
    const fixedStandByRooms = state.standbyRooms.filter(
      (standbyRoomId) => standbyRoomId !== roomId
    );

    if (!!currentRoomInfo) {
      const finalEventMessage:
        | string
        | null = ChatUtil.convertFinalEventMessage(
        action.info.type,
        action.info.sentMessageJson || action.info.sentMessage
      );

      const roomInfo = {
        ...currentRoomInfo,
        finalEventType: info.type,
        finalEventDate: info.sendDate,
        finalEventMessage,
        finalEventSeq: info.seq
      } as RoomInfo;

      return {
        ...state,
        rooms: adapterRoom.upsertOne(roomInfo, { ...state.rooms }),
        standbyRooms: fixedStandByRooms
      };
    } else {
      return state;
    }
  })
);
