import { Selector, createSelector } from '@ngrx/store';
import { EntityState, createEntityAdapter } from '@ngrx/entity';
import { StatusBulkInfo } from '@ucap/protocol-status';

export interface StatusBulkInfoState extends EntityState<StatusBulkInfo> {}

export interface State {
  statusBulkInfo: StatusBulkInfoState;
  statusBulkInfoProcessing: boolean;
}

export const adapterStatusBulkInfo = createEntityAdapter<StatusBulkInfo>({
  selectId: (statusBulkInfo) => statusBulkInfo.userSeq
});

const statusBulkInfoInitialState: StatusBulkInfoState = adapterStatusBulkInfo.getInitialState(
  {}
);

export const initialState: State = {
  statusBulkInfo: statusBulkInfoInitialState,
  statusBulkInfoProcessing: false
};

const {
  selectAll: ngeSelectAllStatusBulkInfo,
  selectEntities: ngeSelectEntitiesStatusBulkInfo
} = adapterStatusBulkInfo.getSelectors();

export function selectors<S>(selector: Selector<any, State>) {
  const selectStatusBulkInfo = createSelector(
    selector,
    (state: State) => state.statusBulkInfo
  );

  return {
    selectAllStatusBulkInfo: createSelector(
      selectStatusBulkInfo,
      ngeSelectAllStatusBulkInfo
    ),
    selectEntitiesStatusBulkInfo: createSelector(
      selectStatusBulkInfo,
      ngeSelectEntitiesStatusBulkInfo
    ),
    selectStatusBulkInfo: (userSeq: number) =>
      createSelector(
        selectStatusBulkInfo,
        ngeSelectEntitiesStatusBulkInfo,
        (_, entities) => (!!entities ? entities[userSeq] : undefined)
      ),
    statusBulkInfoProcessing: createSelector(
      selector,
      (state: State) => state.statusBulkInfoProcessing
    )
  };
}
