import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { PresenceType } from '@ucap/core';
import { UserInfoSS } from '@ucap/protocol-query';
import { StatusBulkInfo } from '@ucap/protocol-status';

import { PresenceUtil } from '../utils/presence.util';

@Component({
  selector: 'ucap-organization-profile-image-01',
  templateUrl: './profile-image-01.component.html',
  styleUrls: ['./profile-image-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileImage01Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoSS;

  @Input()
  presenceInfo: StatusBulkInfo;

  @Input()
  profileImageRoot: string;

  @Input()
  defaultProfileImage: string;

  @Output()
  openProfile: EventEmitter<UserInfoSS> = new EventEmitter();

  PresenceType = PresenceType;
  PresenceUtil = PresenceUtil;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickOpenProfile(event: MouseEvent): void {
    event.preventDefault();
    event.stopPropagation();

    this.openProfile.emit(this.userInfo);
  }
}
