import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';
import { ucapAnimations } from '../animations';

export interface FloatActionButton {
  /** Meterial Icon type */
  icon: string;
  tooltip?: string;
  divisionType?: string;
}

@Component({
  selector: 'ucap-float-action-button',
  templateUrl: './float-action-button.component.html',
  styleUrls: ['./float-action-button.component.scss'],
  animations: ucapAnimations
})
export class FloatActionButtonComponent implements OnInit {
  @Input()
  buttons: FloatActionButton[] = [];

  @Output()
  buttonClick = new EventEmitter<{
    btn: FloatActionButton;
  }>();

  // tslint:disable-next-line: variable-name
  _buttons = [];
  fabTogglerState = 'inactive';

  constructor() {}

  ngOnInit() {}

  showItems() {
    this.fabTogglerState = 'active';
    this._buttons = this.buttons;
  }

  hideItems() {
    this.fabTogglerState = 'inactive';
    this._buttons = [];
  }

  getTooltip(btn: FloatActionButton) {
    if (!!btn && !!btn.tooltip && btn.tooltip.trim().length > 0) {
      return btn.tooltip;
    } else {
      return '';
    }
  }

  onToggleFab() {
    if (this.buttons.length > 1) {
      this._buttons.length ? this.hideItems() : this.showItems();
    } else if (this.buttons.length === 1) {
      this.buttonClick.emit({ btn: this.buttons[0] });
    }
  }

  onClickButton(btn: FloatActionButton) {
    this.hideItems();
    this.buttonClick.emit({ btn });
  }
}
