import { BehaviorSubject, merge, Observable, Subject } from 'rxjs';
import { map, share, takeUntil } from 'rxjs/operators';

import { CollectionViewer, DataSource } from '@angular/cdk/collections';
import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';
import { FlatTreeControl } from '@angular/cdk/tree';

import { MatTreeFlattener } from '@angular/material/tree';

export class VirtualScrollTreeFlatDataSource<T, F> extends DataSource<F> {
  expandedDataSubject = new BehaviorSubject<F[]>([]);
  expandedData$: Observable<F[]>;

  // tslint:disable-next-line: variable-name
  private _flattenedDataSubject = new BehaviorSubject<F[]>([]);
  // tslint:disable-next-line: variable-name
  private _connectSubject: Subject<F[]>;

  // tslint:disable-next-line: variable-name
  private _destroySubject = new Subject<boolean>();

  // tslint:disable-next-line: variable-name
  private _rangeSubject: BehaviorSubject<{
    start: number;
    end: number;
  }>;

  // tslint:disable-next-line: variable-name
  private _cdkVirtualScrollViewport: CdkVirtualScrollViewport;
  set cdkVirtualScrollViewport(
    cdkVirtualScrollViewport: CdkVirtualScrollViewport
  ) {
    if (!cdkVirtualScrollViewport) {
      return;
    }
    this._cdkVirtualScrollViewport = cdkVirtualScrollViewport;
    this._rangeSubject = new BehaviorSubject<{
      start: number;
      end: number;
    }>({
      start: 0,
      end: 1
    });

    cdkVirtualScrollViewport.renderedRangeStream
      .pipe(takeUntil(this._destroySubject))
      .subscribe((range) => {
        this._rangeSubject.next({
          start: range.start,
          end: range.end
        });
        if (!!this._connectSubject) {
          this._connectSubject.next(
            this.expandedDataSubject.value.slice(
              this._rangeSubject.value.start,
              this._rangeSubject.value.end
            )
          );
        }
      });
  }

  // tslint:disable-next-line: variable-name
  private _data: BehaviorSubject<T[]>;
  get data() {
    return this._data.value;
  }
  set data(value: T[]) {
    this._data.next(value);
    this._flattenedDataSubject.next(
      this._treeFlattener.flattenNodes(this.data)
    );
    this._treeControl.dataNodes = this._flattenedDataSubject.value;
  }

  constructor(
    // tslint:disable-next-line: variable-name
    private _treeControl: FlatTreeControl<F>,
    // tslint:disable-next-line: variable-name
    private _treeFlattener: MatTreeFlattener<T, F>,
    initialData: T[] = []
  ) {
    super();
    this._data = new BehaviorSubject<T[]>(initialData);
    this.expandedData$ = this.expandedDataSubject.asObservable().pipe(share());
  }

  connect(collectionViewer: CollectionViewer): Observable<F[]> {
    this._destroySubject = new Subject<boolean>();

    this._connectSubject = new Subject<F[]>();

    merge(
      collectionViewer.viewChange,
      this._treeControl.expansionModel.changed,
      this._flattenedDataSubject
    )
      .pipe(
        takeUntil(this._destroySubject),
        map(() => {
          this.expandedDataSubject.next(
            this._treeFlattener.expandFlattenedNodes(
              this._flattenedDataSubject.value,
              this._treeControl
            )
          );

          return !this._rangeSubject
            ? this.expandedDataSubject.value
            : this.expandedDataSubject.value.slice(
                this._rangeSubject.value.start,
                this._rangeSubject.value.end
              );
        })
      )
      .subscribe((datas) => {
        this._connectSubject.next(datas);
        if (!!this._cdkVirtualScrollViewport) {
          this._cdkVirtualScrollViewport.checkViewportSize();
        }
      });

    return this._connectSubject.asObservable();
  }

  disconnect() {
    if (!!this._connectSubject) {
      this._connectSubject.next();
      this._connectSubject.unsubscribe();
    }

    if (!!this._destroySubject) {
      this._destroySubject.complete();
    }
  }
}
