import { Subject, fromEvent } from 'rxjs';
import { debounceTime, takeUntil } from 'rxjs/operators';

import { Directive, OnInit, OnDestroy, Self, Inject } from '@angular/core';

import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';

@Directive({
  // tslint:disable-next-line: directive-selector
  selector: 'cdk-virtual-scroll-viewport'
})
export class CdkVirtualScrollViewportPatchDirective
  implements OnInit, OnDestroy {
  protected destroySubject: Subject<boolean>;

  constructor(
    @Self()
    @Inject(CdkVirtualScrollViewport)
    private readonly viewportComponent: CdkVirtualScrollViewport
  ) {}

  ngOnInit() {
    this.destroySubject = new Subject();

    fromEvent(document.defaultView, 'resize')
      .pipe(takeUntil(this.destroySubject), debounceTime(10))
      .subscribe(() => {
        this.viewportComponent.checkViewportSize();
      });
  }

  ngOnDestroy() {
    if (!!this.destroySubject) {
      this.destroySubject.next();
      this.destroySubject.complete();
    }
  }
}
